#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Site-Specific Archiver Base Class
from __future__ import print_function
from __future__ import absolute_import

import os


class BaseSiteArchiver(object):
    name = 'base'
    def __init__(self, options):
        if self.name == 'base':
            raise Exception('BaseSiteArchiver must be subclassed!')
        self.threads = {}
        self.options = options
        self.base_thread_dir = os.path.join(options.base_dir, '{}/{{board}}/{{thread}}/'.format(self.name))

    def url_valid(self, url):
        """Return true if the given URL is for my site."""
        raise Exception('you must override this method')

    def add_thread(self, url):
        """Try to add the given thread to our internal list."""
        raise Exception('you must override this method')

    def download_threads(self):
        """Download all the threads we currently hold."""
        # we iterate over a copy of self.threads because download_thread
        # deletes it from there if thread 404's
        for thread_id in dict(self.threads):
            self._download_thread(self.threads[thread_id])

    @property
    def existing_threads(self):
        """Return how many threads we have and are downloading."""
        return len(self.threads)

    def _download_thread(self, thread_info):
        """Download the given thread, from the thread info."""
        raise Exception('you must override this method')


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# BASC Imageboard Archiver Utilities
from __future__ import print_function
from __future__ import absolute_import

import os
import re
import json
import codecs

import requests


def mkdirs(path):
    """Make directory, if it doesn't exist."""
    if not os.path.exists(path):
        os.makedirs(path)


def download_file(local_filename, url, clobber=False):
    """Download the given file. Clobber overwrites file if exists."""
    dir_name = os.path.dirname(local_filename)
    mkdirs(dir_name)

    if clobber or not os.path.exists(local_filename):
        i = requests.get(url)

        # if not exists
        if i.status_code == 404:
            print('Failed to download file:', local_filename, url)
            return False

        # write out in 1MB chunks
        chunk_size_in_bytes = 1024*1024  # 1MB
        with open(local_filename, 'wb') as local_file:
            for chunk in i.iter_content(chunk_size=chunk_size_in_bytes):
                local_file.write(chunk)

    return True


def download_json(local_filename, url, clobber=False):
    """Download the given JSON file, and pretty-print before we output it."""
    if download_file(local_filename, url, clobber):
        # read original json
        original_data = json.loads(open(local_filename).read())

        # write reformatted json
        with open(local_filename, 'w') as json_file:
            json_file.write(json.dumps(original_data, sort_keys=True, indent=2, separators=(',', ': ')))


def file_replace(local_filename, pattern, replacement):
    """Regex replace in the given file."""
    # can't use fileinput lib here because utf-8
    temp_new_filename = '{}-temporary'.format(local_filename)
    with codecs.open(local_filename, 'r', encoding='utf-8') as fi:
        with codecs.open(temp_new_filename, 'w', encoding='utf-8') as fo:
            for line in fi:
                fo.write(re.sub(pattern, replacement, line))
    os.remove(local_filename)
    os.rename(temp_new_filename, local_filename)


def timestamp():
    """Return a timestamp for right now."""
    now = time.time()
    localtime = time.localtime(now)
    return time.strftime('%Y-%m-%d %H:%M:%S', localtime)



#!/usr/bin/env python
# coding: utf-8
import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

with open('README.rst') as file:
    long_description = file.read()


setup(
    name='BASC-Archiver',
    version='0.8.4',
    description='Makes a complete archive of imageboard threads including images, HTML, and JSON.',
    long_description=long_description,
    author='Lawrence Wu <sagnessagiel@gmail.com>, Daniel Oaks <daniel@danieloaks.net>',
    author_email='sagnessagiel@gmail.com',
    url='https://github.com/bibanon/BASC-Archiver',
    scripts=['thread-archiver', '4chan-thread-archiver'],
    packages=['basc_archiver', 'basc_archiver.sites'],
    package_dir={
        'basc_archiver': 'basc_archiver',
        'basc_archiver.sites': 'basc_archiver/sites',
    },
    install_requires=['requests', 'docopt==0.5.0', 'BASC-py4chan'],
    keywords='4chan downloader images json dump',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ]
)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# BASC Imageboard Archiver
from __future__ import print_function
from __future__ import absolute_import

from .sites import default_archivers

version = '0.8.4'
_default_base_dir = './archive'


class Options:
    """Holds Archiver options."""
    def __init__(self, base_dir, use_ssl=False,
                 silent=False, verbose=False, delay=2,
                 skip_thumbs=False, thumbs_only=False,
                 follow_child_threads=False, follow_to_other_boards=False):
        self.base_dir = base_dir
        self.use_ssl = use_ssl
        self.silent = silent
        self.verbose = verbose
        self.delay = delay          # wait 2 seconds by default
        self.skip_thumbs = skip_thumbs
        self.thumbs_only = thumbs_only
        self.follow_child_threads = follow_child_threads
        self.follow_to_other_boards = follow_to_other_boards


class Archiver:
    """Archives the given imageboard threads."""
    def __init__(self, options=None):
        if options is None:
            options = Options(_default_base_dir)
        self.options = options

        # add our default site-specific archivers
        self.archivers = []
        for archiver in default_archivers:
            self.archivers.append(archiver(self.options))
        
    def add_thread(self, url):
        """Archive the given thread if possible"""
        url_archived = False
        for archiver in self.archivers:
            if archiver.url_valid(url):
                archiver.add_thread(url)
                url_archived = True
        
        if url_archived:
            return True
        else:
            print('We could not find a valid archiver for:', url)
            return False
    
    def download_threads(self):
        """Download all the threads we currently hold."""
        for archiver in self.archivers:
            archiver.download_threads()

    @property
    def existing_threads(self):
        """Return how many threads exist."""
        threads = 0
        for archiver in self.archivers:
            threads += archiver.existing_threads
        return threads
